/*
Copyright (c) 2013 General Development Systems

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "insd.hh"

#include <fstream>
#include <iostream>

namespace insd {

pid_t PidFileHolder::conflictExists(std::string filepath) {
//static function for checking that a pidfile is not already there. 
//If filepath (the pidfile) is there already, returns the PID stored in the existing
//pidfile, else returns zero.
//Precondition: undefined behaviour if filepath is not a pidfile.
    std::ifstream pidFileStream;
    pidFileStream.open(filepath.c_str());
    if(!pidFileStream.is_open()) 
        return 0; //no conflicting pidfile exists
    pid_t thePid; 
    pidFileStream >> thePid;
    pidFileStream.close();
    return thePid;
}


PidFileHolder::PidFileHolder(std::string filepath) {
//create pidfile, or throw exception if already exists.
    this->filepath = filepath;
    pid_t p;

    if((p=conflictExists(filepath)))
        throw(ExistingConflictException(p));

    //create the pidfile
    p = getpid();
    std::ofstream pidFileStream;
    pidFileStream.open(filepath.c_str());
    pidFileStream << p << std::endl; 
    pidFileStream.close();
}

PidFileHolder::~PidFileHolder() {
    if(unlink(filepath.c_str()) == -1)
        std::cerr << "insd: could not delete pidfile!" << std::endl;
}

}

