/*
Copyright (c) 2013 General Development Systems

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "insd.hh"

#include <sys/un.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>

namespace insd {

Connection* clientConnect(std::string sockfile) {
    signal(SIGPIPE, SIG_IGN); //ignore SIGPIPEs which otherwise crash the client/server silently.

    int clientFD;
    if((clientFD = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
        throw std::system_error(errno, std::generic_category(), "insd"); //could not allocate new socket descriptor
    }

    struct sockaddr_un remote;
    remote.sun_family = AF_UNIX;
    strcpy(remote.sun_path, sockfile.c_str());
    if(connect(clientFD, (struct sockaddr *)&remote, sizeof(struct sockaddr_un)) == -1) {
		throw(std::system_error(errno, std::generic_category(), "insd: could not connect to server socket"));
    }

    return new Connection(clientFD, nullptr);

}

}

